
#  --------------------------------------------------------------------------------
#  Izok: A modeling system similar to GAMS for pyomo.
#  Copyright 2024,  Andrey Sergeevich Storozhenko, All rights reserved.
#  This software is distributed under the 3-clause BSD License.
#  --------------------------------------------------------------------------------

# parser error_id const
PARSER_EXPECT_EXCEPTION = 1
PARSER_NOT_UNARY_OPERATOR = 2
PARSER_MATCH_RULE_EXCEPTION = 3
PARSER_BAD_CHARS = 4
PARSER_OBJ_SENSE_ERROR = 5
PARSER_DOMAIN_DEFINITION_ERROR = 6

# generation error
GEN_TYPE_CONVERT_ERROR = 7
GEN_FUNCTION_CALL_ERROR = 8
GEN_PROPERTY_NAME_ERROR = 9
GEN_OPERATOR_ERROR = 10
GEN_ESP_NONE_ERROR = 11
GEN_INDEXING_ERROR = 12
